@@ -5,6 +5,7 @@
 #------------------------------------------------------------------------------
 
 # Stdlib
+from __future__ import absolute_import
 import datetime
 
 # Recordpack
@@ -14,10 +15,10 @@ from recordpack.be import BE
 from recordpack.typecast import cast_to_date
 
 # Demo
-import ui
-import column_filter
-from proxy import PersonListProxy
-from models import Person
+from . import ui
+from . import column_filter
+from .proxy import PersonListProxy
+from .models import Person
 
 #------------------------------------------------------------------------------
 # Demonstrative action packs
