@@ -5,22 +5,24 @@
 #------------------------------------------------------------------------------
 
 # 3rdparty
+from __future__ import absolute_import
 from django.conf import urls
+from django.conf.urls import url
+
 from m3_ext.ui import app_ui
 from m3.actions import ControllerCache
 
 # Demo
-import actions
-import controller
+from . import actions
+from . import controller
 
 #------------------------------------------------------------------------------
 # Action packs registration
 #------------------------------------------------------------------------------
 
 def register_urlpatterns():
-    return urls.defaults.patterns(
-        '',
-        ('^', controller.action_controller.process_request),
+    return (
+        url('^', controller.action_controller.process_request),
     )
 
 
