@@ -5,12 +5,15 @@
 #------------------------------------------------------------------------------
 
 # Stdlib
+from __future__ import absolute_import
 import datetime
 
 # 3rdparty
 from django.db import models
 
 # Recordpack
+from m3_django_compat import get_model
+
 from recordpack.be import BE
 from recordpack.typecast import mute_typecast
 
@@ -37,7 +40,7 @@ def person_birthday(date_string):
 
 def person_gender(gender_string):
     _filter = None
-    person_model = models.get_model('demo', 'Person')
+    person_model = get_model('demo', 'Person')
     for index, gender in person_model.GENDERS_LIST:
         if gender_string in gender:
             _filter |= BE('gender', BE.EQ, index)
