@@ -7,9 +7,12 @@ u""" Module description.
 #------------------------------------------------------------------------------
 
 # 3rdparty
+from __future__ import absolute_import
 from django.db import models
 
 # Recordpack
+from m3_django_compat import get_model
+
 from recordpack.provider import BaseProxy
 
 #------------------------------------------------------------------------------
@@ -18,6 +21,6 @@ from recordpack.provider import BaseProxy
 
 class PersonListProxy(BaseProxy):
     def load(self, root):
-        person_model = models.get_model('demo', 'Person')
+        person_model = get_model('demo', 'Person')
         self.from_root()
         self.gender = person_model.GENDERS[self.gender]
