@@ -5,22 +5,38 @@
 #------------------------------------------------------------------------------
 
 # Stdlib
-from abc import ABCMeta, abstractmethod
-from itertools import imap, ifilter
+from __future__ import absolute_import
+import six
+import warnings
+
+from abc import (
+    ABCMeta,
+    abstractmethod,
+)
 from operator import itemgetter
+from six.moves import (
+    map,
+    filter,
+)
 
 # 3rdparty
-from m3.actions import OperationResult
-from django.db.models import Count, Sum
-from django.utils.translation import ugettext as _
-from m3.db import ObjectState
+from django.db.models import (
+    Count,
+    Sum,
+)
+from django.db.models.constants import LOOKUP_SEP
 
 # Recordpack
-from .be import BE
-from .exceptions import (ObjectDoesNotExists, MultipleObjects)
-from .helpers import init_component
-from .proxy import BaseProxy
 from . import signals
+from .be import BE
+from .exceptions import (
+    ObjectDoesNotExists,
+    MultipleObjects,
+)
+from .helpers import (
+    init_component,
+    AttributedDict,
+)
 
 #------------------------------------------------------------------------------
 # Metadata
@@ -37,6 +53,13 @@ __docformat__ = 'restructuredtext'
 SORT_ASC = 'ASC'
 SORT_DESC = 'DESC'
 
+# представление для полей выбора из справочника
+FIELD_VIEW = 'field'
+# представление для карточки
+CARD_VIEW = 'card'
+# представление для списка
+LIST_VIEW = 'list'
+
 
 #------------------------------------------------------------------------------
 # Provider classes
@@ -73,8 +96,16 @@ class QueryObject(object):
         #: `['field_a', '-field_b', 'field_c', ..]`
         self.sorting = []
 
-        #: Cвязанные объекты (будут добавлены как select_related)
-        self.related = []
+        self.select_related = []
+
+        self.prefetch_related = []
+
+        # Список only-полей для queryset.only()
+        self.only = []
+
+        # Список наименований полей, значения для которых требуется получить.
+        # Использует QuerySet.values()
+        self.values = []
 
         #: Описание итогов в формате:
         #: `{'field': 'aggregate_operation', ..}`
@@ -84,12 +115,45 @@ class QueryObject(object):
         #: будет передан всем прокси-объектам
         self.context = None
 
+        # Запрашиваемое представление
+        # может использоваться для подстановки proxy
+        self.view = None
+
+        # Признак выполнения distinct в запросе данных
+        # (накладывает дополнительную нагрузку на сервер БД)
+        # но иногда без него никак
+        self.distinct = False
+
         init_component(self, **kwargs)
 
+        # Удалить, когда будет удалено свойство related
+        self.select_related = self.select_related or kwargs.get('related', [])
 
-class BaseRecordProvider(object):
+    # Удалить, когда будет удален параметр related при использовании объектов
+    # класса
+    @property
+    def related(self):
+        warnings.warn(
+            '`related` is deprecated. '
+            'Use `select_related` instead.',
+            DeprecationWarning)
+
+        return self.select_related
+
+    # Удалить, когда будет удален параметр related при использовании объектов
+    # класса
+    @related.setter
+    def related(self, related):
+        warnings.warn(
+            '`related` is deprecated. '
+            'Use `select_related` instead.',
+            DeprecationWarning)
+
+        self.select_related = related
+
+
+class BaseRecordProvider(six.with_metaclass(ABCMeta, object)):
     u""" Базовый абстрактный класс провайдера данных. """
-    __metaclass__ = ABCMeta
 
     def __init__(self, **kwargs):
         #: Источник данных
@@ -124,7 +188,26 @@ class BaseRecordProvider(object):
 class DjangoModelProvider(BaseRecordProvider):
     u""" Базовый провайдер для моделей Django. """
     
-    def __init__(self, data_source, key_field='id', related=None):
+    def __init__(
+        self,
+        data_source,
+        key_field='id',
+        related=(),
+        select_related=None,
+        prefetch_related=None
+    ):
+        """
+        Инициализация провайдера моделей
+
+        :param data_source: модель-источник данных
+        :param key_field: имя поля первичного ключа
+        :param related: deprecated список полей внешних ключей для
+            select_related. Устаревший вариант, в дальнейшем будет удален.
+            Взамен использовать select_related.
+        :param select_related: список полей внешних ключей на связанные таблицы
+        :param prefetch_related: список обратных внешних ключей на связанные
+            таблицы.
+        """
         super(DjangoModelProvider, self).__init__()
         
         #: Источник данных.
@@ -133,9 +216,9 @@ class DjangoModelProvider(BaseRecordProvider):
         #: Поле модели в :attr:`data_source`, которое является 
         #: первичным ключом.
         self.key_field = key_field
-        
-        #: Связные модели, логика та что и у :meth:`QuerySet.select_related`
-        self.related = related
+
+        self.select_related = select_related or related
+        self.prefetch_related = prefetch_related
         
         #: Агрегаторы данных для итоговой строки
         self.aggregators = {
@@ -143,6 +226,16 @@ class DjangoModelProvider(BaseRecordProvider):
             'count': _aggregate_count,
         }
 
+    # Удалить после отказа от related в кодовой базе
+    @property
+    def related(self):
+        return self.select_related
+
+    # Удалить после отказа от related в кодовой базе
+    @related.setter
+    def related(self, related):
+        self.select_related = related
+
     def _create_filter_conditions(self, filter_):
         u""" Получение фильтра для модели по переданному выражению
         фильтра (BE).
@@ -203,17 +296,33 @@ class DjangoModelProvider(BaseRecordProvider):
         """
         query_object = query_object if query_object else QueryObject()
         try:
-            record = self.get_manager(query_object).get((self.key_field, key))
+            manager = self.get_manager(query_object)
+
+            select_related = self._get_select_related_fields(
+                query_object=query_object,
+            )
+            if select_related:
+                manager = manager.select_related(*self.select_related)
+
+            prefetch_related = self._get_prefetch_related_fields(
+                query_object=query_object,
+            )
+            if prefetch_related:
+                manager = manager.prefetch_related(*self.prefetch_related)
+
+            record = manager.get((self.key_field, key))
         except self.data_source.DoesNotExist:
             raise ObjectDoesNotExists(
                 source=self.data_source,
                 field=self.key_field,
-                key=key)
+                key=key
+            )
         except self.data_source.MultipleObjectsReturned:
             raise MultipleObjects(
                 source=self.data_source,
                 field=self.key_field,
-                key=key)
+                key=key
+            )
         else:
             return record
 
@@ -224,9 +333,19 @@ class DjangoModelProvider(BaseRecordProvider):
         """
         return obj
 
+    def _pre_process_records(self, data, context=None):
+        u"""
+        Функция позволяет переопределить или дополнить объекты,
+        которые являетсю результатом работы :meth:`get_records`.
+        """
+        return [self._preprocess_record(obj, context) for obj in data]
+
     def before_query(self, query_object, query):
         return query
 
+    def after_query(self, query_object, data):
+        return data
+
     def get_manager(self, query_object):
         u""" Получение менеджера модели.
         
@@ -238,8 +357,10 @@ class DjangoModelProvider(BaseRecordProvider):
 
         """
         # иногда query_object может быть None
-        if getattr(query_object, 'from_all', None) \
-                and hasattr(self.data_source, 'objects_all'):
+        if (
+            getattr(query_object, 'from_all', None) and
+            hasattr(self.data_source, 'objects_all')
+        ):
             model_manager = self.data_source.objects_all
         else:
             model_manager = self.data_source.objects
@@ -254,11 +375,72 @@ class DjangoModelProvider(BaseRecordProvider):
         # то он и будет возвращен, в ином случае - значение по-умолчанию.
         extract_manager = itemgetter(1)
         model_manager = next(
-            ifilter(None, imap(extract_manager, receivers_results)),
+            filter(None, map(extract_manager, receivers_results)),
             model_manager)
 
         return model_manager
 
+    @staticmethod
+    def get_query_select_related_fields(data):
+        u"""
+        Метод получения представления связей для проброса в select_related.
+        """
+        result = []
+
+        if (
+            hasattr(data, 'query') and
+            getattr(data.query, 'select_related', None)
+        ):
+            def combine_relations(d, attr='', res=[]):
+                if not d:
+                    res.append(attr)
+
+                attr = attr and attr + LOOKUP_SEP or ''
+
+                for k, v in d.items():
+                    combine_relations(v, attr + k)
+
+                return res
+
+            result = combine_relations(getattr(data.query, 'select_related'))
+
+        return result
+
+    def _get_select_related_fields(self, data=None, query_object=None):
+        u""" Возвращает список атрибутов для добавления в select_related. """
+        all_related = set()
+
+        if query_object and query_object.select_related:
+            all_related.update(query_object.select_related)
+
+        if self.select_related:
+            all_related.update(self.select_related)
+
+        if data is not None:
+            all_related.update(
+                self.get_query_select_related_fields(data)
+            )
+
+        return all_related
+
+    # Удалить после внедрения select_related и prefetch_related
+    def _related_fields(self, *args, **kwargs):
+        return self._get_select_related_fields(*args, **kwargs)
+
+    def _get_prefetch_related_fields(self, data=None, query_object=None):
+        u"""
+        Возвращает список атрибутов для добавления в prefetch_related
+        """
+        all_related = set()
+
+        if self.prefetch_related:
+            all_related.update(self.prefetch_related)
+
+        if query_object and query_object.prefetch_related:
+            all_related.update(query_object.prefetch_related)
+
+        return all_related
+
     def get_records(self, query_object):
         u"""
         Получение записей, которые удовлетворяют условию из *query_object*.
@@ -278,15 +460,24 @@ class DjangoModelProvider(BaseRecordProvider):
         if filter_:
             data = data.filter(filter_)
 
-        search_filter = self._create_filter_conditions(query_object.quick_filter)
+        search_filter = self._create_filter_conditions(
+            query_object.quick_filter,
+        )
         if search_filter:
             data = data.filter(search_filter)
 
         # Связанные записи
         # -------------------------------------------------------------
-        all_related = self._related_fields(data, query_object)
-        if all_related:
-            data = data.select_related(*all_related)
+        select_related = self._get_select_related_fields(data, query_object)
+        if select_related:
+            data = data.select_related(*select_related)
+
+        prefetch_related = self._get_prefetch_related_fields(
+            data,
+            query_object,
+        )
+        if prefetch_related:
+            data = data.prefetch_related(*prefetch_related)
 
         # Сортировка
         # -------------------------------------------------------------
@@ -298,7 +489,16 @@ class DjangoModelProvider(BaseRecordProvider):
 
         # Избавление от дубликатов,
         # которые могут появится при наличии множества JOIN'ов
-        data = data.distinct()
+        if query_object.distinct:
+            data = data.distinct()
+
+        values = query_object.values
+        if values:
+            data = data.values(*values)
+
+        # добавим поддержку only
+        if query_object.only:
+            data = data.only(*query_object.only)
 
         # Обработка запроса перед выборкой
         # -------------------------------------------------------------
@@ -306,14 +506,34 @@ class DjangoModelProvider(BaseRecordProvider):
 
         # Срез
         # -------------------------------------------------------------
-        total = data.count()
         if query_object.end:
-            res_data = data.all()[query_object.begin:query_object.end]
+            begin = max(query_object.begin, 0)
+            end = query_object.end
+            res_data = data.all()[begin:end]
+
+            # Если после пагинации БД отдала количество записей меньшее, чем
+            # мы запросили, дополнительный count-запрос нам не потребуется,
+            # т.к. это значит, что мы запросили данные с последней "страницы".
+            # Но если БД отдала 0 записей, это не значит ничего, т.к. мы могли
+            # запросить записи с 100 по 125, а всего их 80. Исключение - если
+            # мы запрашиваем записи начиная с нулевой.
+            rows_on_page_expected = end - begin
+            rows_on_page_got = len(res_data)
+            if (0 < rows_on_page_got < rows_on_page_expected or
+                    begin == 0 and rows_on_page_got == 0):
+                total = begin + rows_on_page_got
+            else:
+                total = self.__get_total(data)
         else:
             res_data = data.all()
+            total = self.__get_total(data)
+
+        # Обработка запроса после выборки
+        # -------------------------------------------------------------
+        res_data = self.after_query(query_object, res_data)
 
         context = query_object.context
-        result = [self._preprocess_record(obj, context) for obj in res_data]
+        result = self._pre_process_records(res_data, context)
 
         # Подсчет итоговой строки
         # -------------------------------------------------------------
@@ -324,27 +544,6 @@ class DjangoModelProvider(BaseRecordProvider):
         else:
             return {'rows': result, 'total': total}
 
-    def _related_fields(self, data, query_object):
-        u""" Возвращает список атрибутов для добавления в select_related. """
-        all_related = set()
-
-        if query_object.related:
-            all_related.update(query_object.related)
-
-        if self.related:
-            all_related.update(self.related)
-
-        if hasattr(data, 'query') and data.query.select_related:
-            def fn(d, attr='', res=[]):
-                if not d:
-                    res.append(attr)
-                attr = attr and attr + '___' or ''
-                for k, v in d.items():
-                    fn(v, attr + k)
-                return res
-            all_related.update(fn(data.query.select_related))
-        return list(all_related)
-
     def validate(self, obj):
         pass
 
@@ -386,11 +585,25 @@ class DjangoModelProvider(BaseRecordProvider):
 
         if expressions:
             total = results.model.objects.filter(
-                pk__in=results.values_list('pk', flat=True)
+                pk__in=list(results.values_list('pk', flat=True))
             ).aggregate(**expressions)
 
         return total
 
+    @staticmethod
+    def __get_total(data):
+        """Общее количество записей. Через SQL count.
+        """
+        # Новая Django не любит когда в extra что-то есть, когда считает count
+        if data.query.extra_select:
+            c = data.extra(select={})
+            c.query.extra_select.clear()
+            total = c.count()
+        else:
+            total = data.count()
+
+        return total
+
 
 class DjangoProxyProvider(DjangoModelProvider):
     u""" Провайдер данных c использованием прокси-объектов.
@@ -414,21 +627,25 @@ class DjangoProxyProvider(DjangoModelProvider):
         self.card_proxy = card_proxy
         self.list_proxy = list_proxy
 
+    def _proxy_factory(self, proxy_class, obj, context=None):
+        """
+        Позволяет переопределить построение прокси для класса.
+        """
+        proxy = proxy_class(root=obj, context=context)
+        proxy.load(obj)
+        return proxy
+
     def _proxy_card_factory(self, obj, context=None):
         u"""
         Позволяет переопределить построение прокси для формы.
         """
-        proxy = self.card_proxy(root=obj, context=context)
-        proxy.load(obj)
-        return proxy
+        return self._proxy_factory(self.card_proxy, obj, context)
 
     def _proxy_list_factory(self, obj, context=None):
         u"""
         Позволяет переопределить построение прокси для списка.
         """
-        proxy = self.list_proxy(root=obj, context=context)
-        proxy.load(obj)
-        return proxy
+        return self._proxy_factory(self.list_proxy, obj, context)
 
     def new_record(self, **kwargs):
         u"""
@@ -445,15 +662,37 @@ class DjangoProxyProvider(DjangoModelProvider):
         Получает экземпляр прокси, построенный по модели Django.
         """
         obj = super(DjangoProxyProvider, self).get_record(key, query_object)
-        if self.card_proxy is not None:
-            context = query_object.context if query_object else None
-            return self._proxy_card_factory(obj, context=context)
+        return self._create_proxy(obj, query_object)
+
+    def get_query_proxy_class(self, query_object=None):
+        proxy_class = None
+        if query_object and query_object.view:
+            if query_object.view == CARD_VIEW:
+                proxy_class = self.card_proxy
+            if query_object.view == LIST_VIEW:
+                proxy_class = self.list_proxy
+        return proxy_class
+
+    def _create_proxy(self, obj, query_object):
+        context = query_object.context if query_object else None
+        # попробуем подобрать прокси по запросу
+        proxy_class = self.get_query_proxy_class(query_object)
+        if proxy_class:
+            obj = self._proxy_factory(proxy_class, obj, context)
+        elif self.card_proxy is not None:
+            obj = self._proxy_card_factory(obj, context=context)
         return obj
 
     def _preprocess_record(self, obj, context=None):
         u"""
         Переопределяет возвращаемый провайдером класс на прокси объект.
         """
+        if isinstance(obj, dict):
+            # Словарь получен путём использования qs.values().
+            # Для обратной совместимости, включаем возможность
+            # работы с элементами этого словаря как с атрибутами объекта.
+            obj = AttributedDict(obj)
+
         if self.list_proxy is not None:
             return self._proxy_list_factory(obj, context=context)
         return obj
@@ -632,6 +871,7 @@ class ObjectListProvider(BaseRecordProvider):
 def _aggregate_sum(field_name):
     return Sum(field_name)
 
+
 def _aggregate_count(field_name):
     return Count(field_name)
 
